@@ -1,15 +1,20 @@
 ## plotTree.boxplot
-## written by Liam J. Revell 2016
+## written by Liam J. Revell 2016, 2021, 2022, 2024
 
 plotTree.boxplot<-function(tree,x,args.plotTree=list(),
-	args.boxplot=list()){
+	args.boxplot=list(),...){
+	tree<-untangle(tree,"read.tree")
 	cw<-reorder(tree)
-	if(!is.list(x)&&class(x)!="formula"){
-		obj<-setNames(
-			lapply(cw$tip.label,function(x,y) y[which(names(y)==x)],
-			y=x),cw$tip.label)
-	} else obj<-x
-	if(class(x)=="formula") 
+	if(inherits(x,"formula")){ 
+		obj<-x
+	} else {
+		if(!is.list(x)){
+			obj<-setNames(
+				lapply(cw$tip.label,function(x,y) y[which(names(y)==x)],
+					y=x),cw$tip.label)
+		} else obj<-x
+	}
+	if(inherits(obj,"formula")) 
 		args.boxplot$formula<-obj else args.boxplot$x<-obj
 	args.boxplot$horizontal<-TRUE
 	args.boxplot$axes<-FALSE
@@ -36,8 +41,13 @@ plotTree.boxplot<-function(tree,x,args.plotTree=list(),
 	args.boxplot<-c(args.boxplot[ii],args.boxplot[-ii])
 	args.boxplot$plot<-FALSE
 	obj<-do.call(boxplot,args.boxplot)
+	N<-ncol(obj$stats)
+	if(hasArg(ylim)) ylim<-list(...)$ylim
+	else ylim<-c(0.5,N+0.5)
+	args.boxplot$xlim<-ylim
 	args.boxplot$plot<-TRUE
 	args.plotTree$tips<-setNames(1:Ntip(cw),obj$names)
+	args.plotTree$ylim<-ylim
 	do.call(plotTree,args.plotTree)
 	par(mar=args.boxplot$mar)
 	ii<-which(names(args.boxplot)%in%c("formula","x"))
@@ -49,21 +59,26 @@ plotTree.boxplot<-function(tree,x,args.plotTree=list(),
 	invisible(obj)
 }
 
+
 ## plotTree.barplot
-## written by Liam J. Revell 2016, 2017, 2018
+## written by Liam J. Revell 2016, 2017, 2018, 2021, 2024
 
 plotTree.barplot<-function(tree,x,args.plotTree=list(),
-	args.barplot=list(), ...){
+	args.barplot=list(),...){
+	tree$tip.label<-gsub(" ","_",tree$tip.label)
+	tree<-untangle(tree,"read.tree")
 	if(hasArg(add)) add<-list(...)$add
 	else add<-FALSE
 	if(hasArg(args.axis)) args.axis<-list(...)$args.axis
 	else args.axis<-list()
 	args.axis$side<-1
 	cw<-reorder(tree)
+	if(is.vector(x)) names(x)<-gsub(" ","_",names(x))
 	if(is.data.frame(x)) x<-as.matrix(x)
 	if(is.matrix(x)){
 		x<-x[cw$tip.label,]
 		x<-t(x)
+		colnames(x)<-gsub(" ","_",colnames(x))
 	}
 	args.barplot$height<-if(is.matrix(x)) x[,cw$tip.label] else x[cw$tip.label]
 	args.barplot$plot<-FALSE
@@ -77,10 +92,14 @@ plotTree.barplot<-function(tree,x,args.plotTree=list(),
 		args.barplot$mar<-c(5.1,0,2.1,1.1)
 	else args.barplot$mar[2]<-0.1
 	obj<-as.matrix(do.call(barplot,args.barplot))
+	if(hasArg(ylim)) ylim<-list(...)$ylim
+	else ylim<-c(min(obj)-mean(args.barplot$space),
+		max(obj)+mean(args.barplot$space))
 	if(dim(obj)[2]==1) obj<-t(obj)
 	args.plotTree$tips<-setNames(colMeans(obj),cw$tip.label)
 	args.barplot$plot<-TRUE
-	args.barplot$ylim<-range(args.plotTree$tips)
+	args.barplot$ylim<-ylim
+	args.plotTree$ylim<-ylim
 	args.plotTree$tree<-cw
 	if(is.null(args.plotTree$mar)) 
 		args.plotTree$mar<-c(5.1,1.1,2.1,0)
@@ -105,7 +124,7 @@ plotTree.barplot<-function(tree,x,args.plotTree=list(),
 }
 
 ## function to plot bars at the tips of a plotted tree
-## written by Liam J. Revell 2014, 2015, 2018
+## written by Liam J. Revell 2014, 2015, 2018, 2019
 
 plotTree.wBars<-function(tree,x,scale=NULL,width=NULL,type="phylogram",
 	method="plotTree",tip.labels=FALSE,col="grey",border=NULL,...){
@@ -127,13 +146,21 @@ plotTree.wBars<-function(tree,x,scale=NULL,width=NULL,type="phylogram",
 			else fsize<-1
 			if(type=="phylogram"){
 				pp<-par("pin")[1]
-				sw<-fsize*(max(strwidth(tree$tip.label,units="inches")))+1.37*fsize*strwidth("W",units="inches")
-				alp<-optimize(function(a,H,sw,pp,d) (a*1.04*(max(H)+d)+sw-pp)^2,H=H,sw=sw,pp=pp,d=d,interval=c(0,1e6))$minimum
+				sw<-fsize*(max(strwidth(tree$tip.label,
+					units="inches")))+1.37*fsize*strwidth("W",
+					units="inches")
+				alp<-optimize(function(a,H,sw,pp,d) 
+					(a*1.04*(max(H)+d)+sw-pp)^2,H=H,sw=sw,
+					pp=pp,d=d,interval=c(0,1e6))$minimum
 				lims<-c(min(H),max(H)+d+sw/alp)
 			} else if(type=="fan"){
 				pp<-par("pin")[1]
-				sw<-fsize*(max(strwidth(tree$tip.label,units="inches")))+1.37*fsize*strwidth("W",units="inches")
-				alp<-optimize(function(a,H,sw,pp,d) (a*2*1.04*(max(H)+d)+2*sw-pp)^2,H=H,sw=sw,pp=pp,d=d,interval=c(0,1e6))$minimum
+				sw<-fsize*(max(strwidth(tree$tip.label,
+					units="inches")))+1.37*fsize*strwidth("W",
+					units="inches")
+				alp<-optimize(function(a,H,sw,pp,d) 
+					(a*2*1.04*(max(H)+d)+2*sw-pp)^2,H=H,sw=sw,pp=pp,
+						d=d,interval=c(0,1e6))$minimum
 				lims<-c(-max(H)-d-sw/alp,max(H)+d+sw/alp)
 			}	
 		}
@@ -149,29 +176,39 @@ plotTree.wBars<-function(tree,x,scale=NULL,width=NULL,type="phylogram",
 		if(type=="phylogram"){
 			fg<-par()$fg
 			if(!is.ultrametric(tree)){
-				plotTree(um,ftype=if(tip.labels) "i" else "off",xlim=c(0,lims[2]),lwd=1,color="transparent",...)
-				for(i in 1:Ntip(tree)) lines(c(max(tip.h),tip.h[i]),rep(i,2),lty="dotted")
+				plotTree(um,ftype=if(tip.labels) "i" else "off",
+					xlim=c(0,lims[2]),lwd=1,color="transparent",...)
+				for(i in 1:Ntip(tree)) lines(c(max(tip.h),
+					tip.h[i]),rep(i,2),lty="dotted")
 				add<-TRUE
 				par(fg="transparent")
 			} else add=FALSE
-			if(method=="plotTree") capture.output(plotTree(tree,ftype=if(tip.labels) "i" else "off",xlim=c(0,lims[2]),add=add,...))
-			else if(method=="plotSimmap") capture.output(plotSimmap(tree,ftype=if(tip.labels) "i" else "off",xlim=c(0,lims[2]),add=add,...))
+			if(method=="plotTree") capture.output(plotTree(tree,
+				ftype=if(tip.labels) "i" else "off",xlim=c(0,lims[2]),
+				add=add,...))
+			else if(method=="plotSimmap") capture.output(plotSimmap(tree,
+				ftype=if(tip.labels) "i" else "off",xlim=c(0,lims[2]),add=add,...))
 			par(fg=fg)
 		} else if(type=="fan"){
 			fg<-par()$fg
 			if(!is.ultrametric(tree)){
-				plotTree(um,type="fan",ftype=if(tip.labels) "i" else "off",xlim=lims,ylim=lims,lwd=1,color="transparent",...)
+				plotTree(um,type="fan",ftype=if(tip.labels) "i" else "off",xlim=lims,ylim=lims,
+					lwd=1,color="transparent",...)
 				um<-get("last_plot.phylo",envir=.PlotPhyloEnv)
 				par(fg="transparent")
-				plotTree(tree,type="fan",ftype=if(tip.labels) "i" else "off",xlim=lims,ylim=lims,lwd=1,color="transparent",add=TRUE,...)
+				plotTree(tree,type="fan",ftype=if(tip.labels) "i" else "off",xlim=lims,
+					ylim=lims,lwd=1,color="transparent",add=TRUE,...)
 				tt<-get("last_plot.phylo",envir=.PlotPhyloEnv)
 				par(fg="black",lty="solid")
 				for(i in 1:Ntip(tree)) lines(c(um$xx[i],tt$xx[i]),c(um$yy[i],tt$yy[i]),lty="dotted")
 				par(fg="transparent")
 				add<-TRUE
-			} 
-			if(method=="plotTree") capture.output(plotTree(tree,type="fan",ftype=if(tip.labels) "i" else "off",xlim=lims,ylim=lims,add=add,...))
-			else if(method=="plotSimmap") capture.output(plotSimmap(tree,type="fan",ftype=if(tip.labels) "i" else "off",xlim=lims,ylim=lims,add=add,...))
+			} else add<-FALSE
+			if(method=="plotTree") capture.output(plotTree(tree,type="fan",
+				ftype=if(tip.labels) "i" else "off",xlim=lims,ylim=lims,add=add,...))
+			else if(method=="plotSimmap") capture.output(plotSimmap(tree,
+				type="fan",ftype=if(tip.labels) "i" else "off",xlim=lims,
+				ylim=lims,add=add,...))
 			par(fg=fg)
 		}
 		obj<-get("last_plot.phylo",envir=.PlotPhyloEnv)
